import 'package:aliyun_oss_platform_interface/aliyun_oss_platform_interface.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:freezed_annotation/freezed_annotation.dart';

export 'oss/export_oss.dart';

const MethodChannel _methodChannel =
    MethodChannel('com.wombat/aliyun_oss_plugin');

class MethodChannelAliyunOSS extends AliyunOSSPlatform {
  MethodChannelAliyunOSS() {
    _methodChannel.setMethodCallHandler(_methodCallhandler);
  }

  final List<MethodCallListener> _methodCallListeners = [];

  void _notifyListeners(void Function(MethodCallListener listener) callback) {
    UnmodifiableListView(_methodCallListeners).forEach((element) {
      if (_methodCallListeners.contains(element)) {
        callback(element);
      }
    });
  }

  _toMap(dynamic data) {
    return (data as Map<dynamic, dynamic>)
        .map((key, value) => MapEntry('$key', value));
  }

  Future<dynamic> _methodCallhandler(MethodCall methodCall) async {
    debugPrint('回调方法: ${methodCall.method}\n参数: ${methodCall.arguments}');
    switch (methodCall.method) {
      case 'upload_init':
        UploadInitData initData =
            UploadInitData.fromJson(_toMap(methodCall.arguments));
        debugPrint("上传初始化: ${initData.toJson()}");
        _notifyListeners((listener) => listener.initListener?.call(initData));
        break;
      case "upload_progress":
        UploadProgressData progressData =
            UploadProgressData.fromJson(_toMap(methodCall.arguments));
        debugPrint("上传进度: ${progressData.toJson()}");
        _notifyListeners(
            (listener) => listener.progressListener?.call(progressData));
        break;
      case "upload_success":
        UploadSuccessData successData =
            UploadSuccessData.fromJson(_toMap(methodCall.arguments));
        debugPrint("上传成功: ${successData.toJson()}");
        _notifyListeners(
            (listener) => listener.successListener?.call(successData));
        break;
      case "upload_fail":
        UploadFailData failData =
            UploadFailData.fromJson(_toMap(methodCall.arguments));
        debugPrint("上传失败: ${failData.toJson()}");
        _notifyListeners((listener) => listener.failListener?.call(failData));
        break;
      case "upload_pause":
        _notifyListeners((listener) =>
            listener.pauseListener?.call(methodCall.arguments["objectKey"]));
        break;
      case "upload_resume":
        _notifyListeners((listener) =>
            listener.resumeListener?.call(methodCall.arguments["objectKey"]));
        break;
    }
  }

  @override
  Future<String> getMd5Str(String filePath) async {
    return (await _methodChannel
        .invokeMethod('getMd5Str', {'filePath': filePath})) as String;
  }

  @override
  init(String endpoint, OSSCredentialProvider provider) {
    return _methodChannel.invokeMethod(
        'init', provider.toJson()..addAll({"endpoint": endpoint}));
  }

  @override
  Future<dynamic> upload(String bucketName) async {
    return _methodChannel.invokeMethod('upload', {"bucketName": bucketName});
  }

  @override
  uploadWithPath(String bucketName, String filePath) {
    return _methodChannel.invokeMethod(
        'uploadWithPath', {"bucketName": bucketName, "filePath": filePath});
  }

  @override
  Future<String?> pauseUpload(String objectKey) {
    return _methodChannel
        .invokeMethod('pause_upload', {'objectKey': objectKey});
  }

  @override
  Future<String?> resumeUpload(String objectKey) {
    return _methodChannel
        .invokeMethod('resume_upload', {'objectKey': objectKey});
  }

  @override
  void addMethodCallListener(MethodCallListener listener) {
    _methodCallListeners.add(listener);
  }

  @override
  void removeMethodCallListener(MethodCallListener listener) {
    _methodCallListeners.remove(listener);
  }
}
